@@ -7,7 +7,6 @@ from __future__ import annotations
 
 import asyncio
 import logging
-import time
 from typing import Callable, cast
 
 from xknx.exceptions import CommunicationError, CouldNotParseKNXIP, IncompleteKNXIPFrame
@@ -72,8 +71,7 @@ class TCPTransport(KNXIPTransport):
         if not raw:
             return
         try:
-            knxipframe = KNXIPFrame()
-            frame_length = knxipframe.from_knx(raw)
+            knxipframe, next_frame_part = KNXIPFrame.from_knx(raw)
         except IncompleteKNXIPFrame:
             self._buffer = raw
             raw_socket_logger.debug(
@@ -82,25 +80,21 @@ class TCPTransport(KNXIPTransport):
             return
         except CouldNotParseKNXIP as couldnotparseknxip:
             knx_logger.debug(
-                "Unsupported KNXIPFrame from %s at %s: %s in %s",
+                "Unsupported KNXIPFrame from %s: %s in %s",
                 self.remote_hpai,
-                time.time(),
                 couldnotparseknxip.description,
                 raw.hex(),
             )
-            if not (frame_length := knxipframe.header.total_length):
-                return
         else:
             knx_logger.debug(
-                "Received from %s at %s:\n%s",
+                "Received from %s: %s",
                 self.remote_hpai,
-                time.time(),
                 knxipframe,
             )
             self.handle_knxipframe(knxipframe, self.remote_hpai)
         # parse data after current KNX/IP frame
-        if len(raw) > frame_length:
-            self.data_received_callback(raw[frame_length:])
+        if next_frame_part:
+            self.data_received_callback(next_frame_part)
 
     async def connect(self) -> None:
         """Connect TCP socket."""
@@ -128,9 +122,8 @@ class TCPTransport(KNXIPTransport):
     def send(self, knxipframe: KNXIPFrame, addr: tuple[str, int] | None = None) -> None:
         """Send KNXIPFrame to socket. `addr` is ignored on TCP."""
         knx_logger.debug(
-            "Sending to %s at %s:\n%s",
+            "Sending to %s: %s",
             self.remote_hpai,
-            time.time(),
             knxipframe,
         )
         if self.transport is None:
