@@ -39,102 +39,101 @@ from .tunnelling_request import TunnellingRequest
 class KNXIPFrame:
     """Class for KNX/IP Frames."""
 
-    def __init__(self) -> None:
+    def __init__(self, header: KNXIPHeader, body: KNXIPBody) -> None:
         """Initialize object."""
-        self.header = KNXIPHeader()
-        self.body: KNXIPBody | None = None
-
-    def init(self, service_type_ident: KNXIPServiceType) -> KNXIPBody:
-        """Init object by service_type_ident. Will instanciate a body object depending on service_type_ident."""
-        self.header.service_type_ident = service_type_ident
-
-        body: KNXIPBody
-        # Core
-        if service_type_ident == KNXIPServiceType.SEARCH_REQUEST:
-            body = SearchRequest()
-        elif service_type_ident == KNXIPServiceType.SEARCH_REQUEST_EXTENDED:
-            body = SearchRequestExtended()
-        elif service_type_ident == KNXIPServiceType.SEARCH_RESPONSE:
-            body = SearchResponse()
-        elif service_type_ident == KNXIPServiceType.SEARCH_RESPONSE_EXTENDED:
-            body = SearchResponseExtended()
-        elif service_type_ident == KNXIPServiceType.DESCRIPTION_REQUEST:
-            body = DescriptionRequest()
-        elif service_type_ident == KNXIPServiceType.DESCRIPTION_RESPONSE:
-            body = DescriptionResponse()
-        elif service_type_ident == KNXIPServiceType.CONNECT_REQUEST:
-            body = ConnectRequest()
-        elif service_type_ident == KNXIPServiceType.CONNECT_RESPONSE:
-            body = ConnectResponse()
-        elif service_type_ident == KNXIPServiceType.CONNECTIONSTATE_REQUEST:
-            body = ConnectionStateRequest()
-        elif service_type_ident == KNXIPServiceType.CONNECTIONSTATE_RESPONSE:
-            body = ConnectionStateResponse()
-        elif service_type_ident == KNXIPServiceType.DISCONNECT_REQUEST:
-            body = DisconnectRequest()
-        elif service_type_ident == KNXIPServiceType.DISCONNECT_RESPONSE:
-            body = DisconnectResponse()
-        # Tunneling
-        elif service_type_ident == KNXIPServiceType.TUNNELLING_REQUEST:
-            body = TunnellingRequest()
-        elif service_type_ident == KNXIPServiceType.TUNNELLING_ACK:
-            body = TunnellingAck()
-        # Routing
-        elif service_type_ident == KNXIPServiceType.ROUTING_INDICATION:
-            body = RoutingIndication()
-        elif service_type_ident == KNXIPServiceType.ROUTING_BUSY:
-            body = RoutingBusy()
-        elif service_type_ident == KNXIPServiceType.ROUTING_LOST_MESSAGE:
-            body = RoutingLostMessage()
-        # Secure
-        elif service_type_ident == KNXIPServiceType.SECURE_WRAPPER:
-            body = SecureWrapper()
-        elif service_type_ident == KNXIPServiceType.SESSION_AUTHENTICATE:
-            body = SessionAuthenticate()
-        elif service_type_ident == KNXIPServiceType.SESSION_REQUEST:
-            body = SessionRequest()
-        elif service_type_ident == KNXIPServiceType.SESSION_RESPONSE:
-            body = SessionResponse()
-        elif service_type_ident == KNXIPServiceType.SESSION_STATUS:
-            body = SessionStatus()
-        elif service_type_ident == KNXIPServiceType.TIMER_NOTIFY:
-            body = TimerNotify()
-        else:
-            raise CouldNotParseKNXIP(
-                f"KNXIPServiceType not implemented: {service_type_ident.name}"
-            )
+        self.header = header
         self.body = body
-        return body
 
     @staticmethod
     def init_from_body(knxip_body: KNXIPBody) -> KNXIPFrame:
         """Return KNXIPFrame from KNXIPBody."""
-        knxipframe = KNXIPFrame()
-        knxipframe.header.service_type_ident = knxip_body.__class__.SERVICE_TYPE
-        knxipframe.body = knxip_body
-        knxipframe.header.set_length(knxip_body)
-        return knxipframe
+        header = KNXIPHeader()
+        header.service_type_ident = knxip_body.__class__.SERVICE_TYPE
+        header.set_length(knxip_body)
+        return KNXIPFrame(header=header, body=knxip_body)
 
-    def from_knx(self, data: bytes) -> int:
-        """Parse/deserialize from KNX/IP raw data."""
-        pos = self.header.from_knx(data)
-        if len(data) < self.header.total_length:
+    @staticmethod
+    def from_knx(data: bytes) -> tuple[KNXIPFrame, bytes]:
+        """
+        Parse/deserialize from KNX/IP raw data.
+
+        Returns a tuple of the KNXIPFrame and the rest of the data.
+
+        Raises IncompleteKNXIPFrame if the data is not enough to parse the KNXIPFrame
+        or CouldNotParseKNXIP on parsing error.
+        """
+        header = KNXIPHeader()
+        pos_body = header.from_knx(data)
+        if len(data) < header.total_length:
             raise IncompleteKNXIPFrame("Incomplete data for KNXIPFrame")
         # limit data to self.header.total_length for streaming socket data
-        self.init(self.header.service_type_ident).from_knx(
-            data[pos : self.header.total_length]
-        )
-        return self.header.total_length
+        raw_body = data[pos_body : header.total_length]
+
+        body: KNXIPBody
+        # Core
+        if header.service_type_ident == KNXIPServiceType.SEARCH_REQUEST:
+            body = SearchRequest()
+        elif header.service_type_ident == KNXIPServiceType.SEARCH_REQUEST_EXTENDED:
+            body = SearchRequestExtended()
+        elif header.service_type_ident == KNXIPServiceType.SEARCH_RESPONSE:
+            body = SearchResponse()
+        elif header.service_type_ident == KNXIPServiceType.SEARCH_RESPONSE_EXTENDED:
+            body = SearchResponseExtended()
+        elif header.service_type_ident == KNXIPServiceType.DESCRIPTION_REQUEST:
+            body = DescriptionRequest()
+        elif header.service_type_ident == KNXIPServiceType.DESCRIPTION_RESPONSE:
+            body = DescriptionResponse()
+        elif header.service_type_ident == KNXIPServiceType.CONNECT_REQUEST:
+            body = ConnectRequest()
+        elif header.service_type_ident == KNXIPServiceType.CONNECT_RESPONSE:
+            body = ConnectResponse()
+        elif header.service_type_ident == KNXIPServiceType.CONNECTIONSTATE_REQUEST:
+            body = ConnectionStateRequest()
+        elif header.service_type_ident == KNXIPServiceType.CONNECTIONSTATE_RESPONSE:
+            body = ConnectionStateResponse()
+        elif header.service_type_ident == KNXIPServiceType.DISCONNECT_REQUEST:
+            body = DisconnectRequest()
+        elif header.service_type_ident == KNXIPServiceType.DISCONNECT_RESPONSE:
+            body = DisconnectResponse()
+        # Tunneling
+        elif header.service_type_ident == KNXIPServiceType.TUNNELLING_REQUEST:
+            body = TunnellingRequest()
+        elif header.service_type_ident == KNXIPServiceType.TUNNELLING_ACK:
+            body = TunnellingAck()
+        # Routing
+        elif header.service_type_ident == KNXIPServiceType.ROUTING_INDICATION:
+            body = RoutingIndication()
+        elif header.service_type_ident == KNXIPServiceType.ROUTING_BUSY:
+            body = RoutingBusy()
+        elif header.service_type_ident == KNXIPServiceType.ROUTING_LOST_MESSAGE:
+            body = RoutingLostMessage()
+        # Secure
+        elif header.service_type_ident == KNXIPServiceType.SECURE_WRAPPER:
+            body = SecureWrapper()
+        elif header.service_type_ident == KNXIPServiceType.SESSION_AUTHENTICATE:
+            body = SessionAuthenticate()
+        elif header.service_type_ident == KNXIPServiceType.SESSION_REQUEST:
+            body = SessionRequest()
+        elif header.service_type_ident == KNXIPServiceType.SESSION_RESPONSE:
+            body = SessionResponse()
+        elif header.service_type_ident == KNXIPServiceType.SESSION_STATUS:
+            body = SessionStatus()
+        elif header.service_type_ident == KNXIPServiceType.TIMER_NOTIFY:
+            body = TimerNotify()
+        else:
+            raise CouldNotParseKNXIP(
+                f"KNXIPServiceType not implemented: {header.service_type_ident.name}"
+            )
+        body.from_knx(raw_body)
+        return KNXIPFrame(header=header, body=body), data[header.total_length :]
 
     def to_knx(self) -> bytes:
         """Serialize to KNX/IP raw data."""
-        if self.body is None:
-            raise CouldNotParseKNXIP("No body defined in KNXIPFrame.")
         return self.header.to_knx() + self.body.to_knx()
 
     def __repr__(self) -> str:
         """Return object as readable string."""
-        return f'<KNXIPFrame {self.header}\n body="{self.body}" />'
+        return f'<KNXIPFrame {self.header} body="{self.body}" />'
 
     def __eq__(self, other: object) -> bool:
         """Equal operator."""
